unit Nodes;

interface

uses
  SysUtils, Classes;

type
  Node = class
    name, cname: string;
    _deprecated: boolean;
  end;

  ParamNode = class(Node)
  end;

  ProcNode = class(Node)
    ReturnType: string;
    ParamList: TList;
    constructor Create(procname, proctype: string);
    destructor Destroy; override;
  end;

  EnumMember = class(Node)
    value: string;
    constructor Create(aname, acname, avalue: string);
  end;

  EnumType = (etUnknown, etEnum, etFlags);

  EnumNode = class(Node)
    enumTyp: EnumType;
    gtypeProc: ProcNode;
    memberList: TList;
    MaxNameLength,
    MaxCNameLength: integer;
    constructor Create;
    destructor Destroy; override;
    procedure AddMember(m: EnumMember);
  end;

  ParameterNode = class(Node)
    atype: string;
    isarray: boolean;
    constructor Create;
  end;

  MethodNode = class(Node)
    return_type: string;
    paramList: TList;
    shared: boolean;
    hidden: boolean;
    preferred: boolean;

    pass_as: string;
    owned: boolean;
    property_name: string;

    procedure AddParam(p: ParameterNode);
    constructor Create;
    destructor Destroy; override;
  end;

  CallbackNode = class(MethodNode);
  SignalNode = class(MethodNode)
    when: string;
  end;
  VMethodNode = class(MethodNode);
  ConstructorNode = class(MethodNode);

  PropNode = class(Node)
    atype: string;
    readable, writeable: boolean;
    child: boolean;
  end;

  PointerNode = class(Node)
    iNode: Node;
    constructor Create(n: Node);
  end;

  InterfaceNode = class(Node)
    methodsList: TList;
    signalsList: TList;
    vmethodsList: TList;
    props: TList;
    iPointer: PointerNode;
    constructor Create;
    destructor Destroy; override;
  end;

  FieldNode = class(Node)
    atype: string;
  end;

  ObjectNode = class(Node)
    fields: TList;
    methodsList: TList;
    signalsList: TList;
    vmethodsList: TList;
    props: TList;
    iPointer: PointerNode;
    parent: string;
    constructors: TList;
    constructor Create;
    destructor Destroy; override;

    procedure AddField(f: FieldNode);
  end;

implementation

constructor ProcNode.Create(procname, proctype: string);
begin
  ParamList := TList.Create;
  name := procname;
  returntype := proctype;
end;

destructor ProcNode.Destroy;
begin
  FreeAndNil(ParamList);
end;

constructor EnumNode.Create;
begin
  memberList := TList.Create;
  MaxNameLength  := 0;
  MaxCNameLength := 0;
end;

destructor EnumNode.Destroy;
begin
  FreeAndNil(memberList);
end;
procedure EnumNode.AddMember(m: EnumMember);
var
  l: integer;
begin
  l := Length(m.name);
  if l > MaxNameLength then MaxNameLength := l;

  l := Length(m.cname);
  if l > MaxCNameLength then MaxCNameLength := l;
  memberList.Add(m);
end;

constructor EnumMember.Create(aname, acname, avalue: string);
begin
  name  := aname;
  value := avalue;
  cname := acname;
end;

{ ParameterNode }

constructor ParameterNode.Create;
begin
  isarray := false;
end;

{ CallbackNode }

procedure MethodNode.AddParam(p: ParameterNode);
begin
  paramList.Add(p)
end;

constructor MethodNode.Create;
begin
  paramList := TList.Create;
end;

destructor MethodNode.Destroy;
begin
  paramList.Free;
  inherited;
end;

{ InterfaceNode }

constructor InterfaceNode.Create;
begin
  methodsList := TList.Create;
  signalsList := TList.Create;
  vmethodsList := TList.Create;
  props := TList.Create;
end;

destructor InterfaceNode.Destroy;
begin
  methodsList.Free;
  signalsList.Free;
  vmethodsList.Free;
  props.Free;
  inherited;
end;

{ PointerNode }

constructor PointerNode.Create(n: Node);
begin
  if n = nil then Exit;
  name := 'P' + n.cname;
  iNode := n;
end;

{ ObjectNode }

procedure ObjectNode.AddField(f: FieldNode);
begin
  fields.Add(f)
end;

constructor ObjectNode.Create;
begin
  fields := TList.Create;
  methodsList := TList.Create;
  signalsList := TList.Create;
  vmethodsList := TList.Create;
  props := TList.Create;
  constructors := TList.Create;
end;

destructor ObjectNode.Destroy;
begin
  FreeAndNil(fields);
  FreeAndNil(methodsList);
  FreeAndNil(signalsList);
  FreeAndNil(vmethodsList);
  FreeAndNil(props);
  FreeAndNil(constructors);
  inherited;
end;

end.